@@ -301,8 +301,15 @@ Kid3Form::Kid3Form(Kid3Application* app, BaseMainWindowImpl* mainWin,
   m_dirListBox->setRootIsDecorated(false);
   m_dirListBox->setModel(m_app->getDirProxyModel());
   m_dirListBox->setSelectionModel(m_app->getDirSelectionModel());
+  m_dirListBox->setMaxNumColumns(TaggedFileSystemModel::NUM_FILESYSTEM_COLUMNS);
+  connect(m_fileListBox, &QAbstractItemView::activated,
+          this, &Kid3Form::fileActivated);
   connect(m_dirListBox, &QAbstractItemView::activated, this,
       &Kid3Form::dirSelected);
+  connect(m_fileListBox, &ConfigurableTreeView::parentActivated,
+          this, &Kid3Form::openParentDirectory);
+  connect(m_dirListBox, &ConfigurableTreeView::parentActivated,
+          this, &Kid3Form::openParentDirectory);
 
   connect(m_app, &Kid3Application::fileRootIndexChanged,
           this, &Kid3Form::setFileRootIndex);
@@ -585,12 +592,14 @@ Kid3Form::Kid3Form(Kid3Application* app, BaseMainWindowImpl* mainWin,
 
   auto sectionActions = new SectionActions(SectionActions::Navigation,
                                            m_fileListBox);
+  m_sectionActions.append(sectionActions);
   connect(sectionActions->previousSectionAction(), &QAction::triggered,
           this, [this]() { setFocusPreviousTag(Frame::Tag_NumValues); });
   connect(sectionActions->nextSectionAction(), &QAction::triggered,
           this, &Kid3Form::setFocusDirList);
 
   sectionActions = new SectionActions(SectionActions::Navigation, m_dirListBox);
+  m_sectionActions.append(sectionActions);
   connect(sectionActions->previousSectionAction(), &QAction::triggered,
           this, &Kid3Form::setFocusFileList);
   connect(sectionActions->nextSectionAction(), &QAction::triggered,
@@ -599,6 +608,7 @@ Kid3Form::Kid3Form(Kid3Application* app, BaseMainWindowImpl* mainWin,
   sectionActions = new SectionActions(SectionActions::Navigation |
                                       SectionActions::Transfer,
                                       m_fileWidget);
+  m_sectionActions.append(sectionActions);
   connect(sectionActions->previousSectionAction(), &QAction::triggered,
           this, &Kid3Form::setFocusDirList);
   connect(sectionActions->nextSectionAction(), &QAction::triggered,
@@ -616,6 +626,7 @@ Kid3Form::Kid3Form(Kid3Application* app, BaseMainWindowImpl* mainWin,
                                   SectionActions::EditSection |
                                   SectionActions::EditElement,
           m_frameTable[tagNr]);
+    m_sectionActions.append(sectionActions);
     connect(sectionActions->previousSectionAction(), &QAction::triggered,
             this, [this, tagNr]() {
       setFocusPreviousTag(tagNr);
@@ -662,6 +673,19 @@ Kid3Form::~Kid3Form()
   m_app->removeFrameEditor(m_mainWin);
 }
 
+/**
+ * Set keyboard shortcuts for section actions.
+ * @param map map of action names to key sequences
+ */
+void Kid3Form::setSectionActionShortcuts(const QMap<QString, QKeySequence>& map)
+{
+  for (SectionActions* sectionActions : m_sectionActions) {
+    sectionActions->setShortcuts(map);
+  }
+  m_fileListBox->setShortcuts(map);
+  m_dirListBox->setShortcuts(map);
+}
+
 /**
  * Handle event when mouse is moved while dragging.
  *
@@ -792,6 +816,45 @@ void Kid3Form::dirSelected(const QModelIndex& index)
   }
 }
 
+/**
+ * File list box item activated.
+ *
+ * @param index selected item
+ */
+void Kid3Form::fileActivated(const QModelIndex& index)
+{
+  if (const FileProxyModel* fileProxyModel =
+      qobject_cast<const FileProxyModel*>(index.model())) {
+    if (fileProxyModel->isDir(index)) {
+      QString dirPath = fileProxyModel->filePath(index);
+      if (!dirPath.isEmpty()) {
+        m_mainWin->updateCurrentSelection();
+        m_mainWin->confirmedOpenDirectory({dirPath});
+      }
+    }
+  }
+}
+
+/**
+ * Open the parent directory of a model index.
+ *
+ * @param index current root index of item view
+ */
+void Kid3Form::openParentDirectory(const QModelIndex& index)
+{
+  if (index.isValid()) {
+    QDir dir(index.data(FileSystemModel::FilePathRole).toString());
+    if (dir.cdUp()) {
+      QString dirPath = dir.absolutePath();
+      if (m_dirListBox && index.model() == m_dirListBox->model()) {
+        m_app->setDirUpIndex(index);
+      }
+      m_mainWin->updateCurrentSelection();
+      m_mainWin->confirmedOpenDirectory({dirPath});
+    }
+  }
+}
+
 /**
  * Enable or disable controls requiring tags.
  * @param tagNr tag number
@@ -921,32 +984,13 @@ void Kid3Form::onFirstDirectoryOpened()
   const GuiConfig& guiCfg = GuiConfig::instance();
   m_app->getFileProxyModel()->sort(guiCfg.fileListSortColumn(),
                                    guiCfg.fileListSortOrder());
-  int firstFileSectionSize = 0;
-  const QHeaderView* fileHeader = m_fileListBox->header();
-  const auto columns = guiCfg.fileListVisibleColumns();
-  for (int column : columns) {
-    m_fileListBox->resizeColumnToContents(column);
-    if (firstFileSectionSize <= 0 && fileHeader) {
-      firstFileSectionSize = fileHeader->sectionSize(column);
-    }
-  }
+  int firstFileSectionSize =
+      m_fileListBox->initializeColumnWidthsFromContents(-1);
   m_fileListBox->scrollTo(m_fileListBox->currentIndex());
-
-  int firstDirSectionSize = 0;
-  QHeaderView* dirHeader = m_dirListBox->header();
-  const auto dirColumns = guiCfg.dirListVisibleColumns();
-  for (int column : dirColumns) {
-    m_dirListBox->resizeColumnToContents(column);
-    if (firstDirSectionSize <= 0 && dirHeader) {
-      firstDirSectionSize = dirHeader->sectionSize(column);
-      if (firstDirSectionSize < firstFileSectionSize) {
-        // The directory column often only contains "." and "..", which results
-        // in a small size. Make it at least as wide as the corresponding
-        // file list column.
-        dirHeader->resizeSection(column, firstFileSectionSize);
-      }
-    }
-  }
+  // The directory column often only contains "." and "..", which results
+  // in a small size. Make it at least as wide as the corresponding
+  // file list column.
+  m_dirListBox->initializeColumnWidthsFromContents(firstFileSectionSize);
 }
 
 /**
@@ -1071,16 +1115,7 @@ void Kid3Form::saveConfig()
       guiCfg.setHideTag(tagNr, m_tagWidget[tagNr]->isHidden());
     }
   }
-  int column;
-  Qt::SortOrder order;
-  m_fileListBox->getSortByColumn(column, order);
-  guiCfg.setFileListSortColumn(column);
-  guiCfg.setFileListSortOrder(order);
-  guiCfg.setFileListVisibleColumns(m_fileListBox->getVisibleColumns());
-  m_dirListBox->getSortByColumn(column, order);
-  guiCfg.setDirListSortColumn(column);
-  guiCfg.setDirListSortOrder(order);
-  guiCfg.setDirListVisibleColumns(m_dirListBox->getVisibleColumns());
+  saveFileAndDirListConfig();
 }
 
 /**
@@ -1115,12 +1150,55 @@ void Kid3Form::readConfig()
     }
   }
   hidePicture(guiCfg.hidePicture());
+  readFileAndDirListConfig();
+}
+
+/**
+ * Save file and directory list columns to the configuration.
+ */
+void Kid3Form::saveFileAndDirListConfig()
+{
+  GuiConfig& guiCfg = GuiConfig::instance();
+  int column;
+  Qt::SortOrder order;
+  m_fileListBox->getSortByColumn(column, order);
+  guiCfg.setFileListSortColumn(column);
+  guiCfg.setFileListSortOrder(order);
+  guiCfg.setFileListVisibleColumns(m_fileListBox->getVisibleColumns());
+  bool customColumWidthsEnabled = m_fileListBox->areCustomColumnWidthsEnabled();
+  guiCfg.setFileListCustomColumnWidthsEnabled(customColumWidthsEnabled);
+  if (customColumWidthsEnabled) {
+    guiCfg.setFileListColumnWidths(m_fileListBox->getColumnWidths());
+  }
+  m_dirListBox->getSortByColumn(column, order);
+  guiCfg.setDirListSortColumn(column);
+  guiCfg.setDirListSortOrder(order);
+  guiCfg.setDirListVisibleColumns(m_dirListBox->getVisibleColumns());
+  customColumWidthsEnabled = m_dirListBox->areCustomColumnWidthsEnabled();
+  guiCfg.setDirListCustomColumnWidthsEnabled(customColumWidthsEnabled);
+  if (customColumWidthsEnabled) {
+    guiCfg.setDirListColumnWidths(m_dirListBox->getColumnWidths());
+  }
+}
+
+/**
+ * Set file and directory list columns from the configuration.
+ */
+void Kid3Form::readFileAndDirListConfig()
+{
+  const GuiConfig& guiCfg = GuiConfig::instance();
   m_fileListBox->sortByColumn(guiCfg.fileListSortColumn(),
                               guiCfg.fileListSortOrder());
   m_fileListBox->setVisibleColumns(guiCfg.fileListVisibleColumns());
+  m_fileListBox->setColumnWidths(guiCfg.fileListColumnWidths());
+  m_fileListBox->setCustomColumnWidthsEnabled(
+        guiCfg.fileListCustomColumnWidthsEnabled());
   m_dirListBox->sortByColumn(guiCfg.dirListSortColumn(),
                              guiCfg.dirListSortOrder());
   m_dirListBox->setVisibleColumns(guiCfg.dirListVisibleColumns());
+  m_dirListBox->setColumnWidths(guiCfg.dirListColumnWidths());
+  m_dirListBox->setCustomColumnWidthsEnabled(
+        guiCfg.dirListCustomColumnWidthsEnabled());
 }
 
 /**
